#if ! defined(__LKT_COMMANDS_H__)
#define __LKT_COMMANDS_H__

#include <lektor/common.h>
#include <lektor/net.h>
#include <lektor/database.h>

#include <stdbool.h>

/**
 * The list of commands implemented by lektor.
 *
 * Refer to the following link for the description of each command:
 * <https://www.musicpd.org/doc/html/protocol.html>
 */

/* Querying lektor's status */
bool command_currentsong(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_status     (struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_stats      (struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);

/* Controlling playback */
bool command_next    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_pause   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_previous(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_play    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_playid  (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_stop    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);

bool command_seek    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_seekid  (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_seekcur (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);

bool command_set_pos(struct lkt_state *srv, int index); /* FIXME: Not used in net/listen.c */

/* The queue */
bool command_add    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX], int priority);
bool command_addid  (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_del    (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_delid  (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_clear  (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_crop   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_move   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_shuffle(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_dump   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_swap   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_swapid (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_flat   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);

bool command_queue_list  (struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_queue_listid(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);

/* The playlists */
bool command_plt_create(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_remove(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_clear (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_rename(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_export(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_import(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_add   (struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_list  (struct lkt_state *srv, size_t, char *args[LKT_MESSAGE_ARGS_MAX], int cont);
bool command_plt_ctx   (struct lkt_state *srv, size_t, char *args[LKT_MESSAGE_ARGS_MAX], int cont);

/* The help */
bool command_help(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX]);

/* Respond to the idle command */
bool command_idle(struct lkt_state *srv, size_t c, struct lkt_command *cmd);

/* Cancel the call to the `idle` command. */
bool command_noidle(struct lkt_state *srv, size_t c);

/* Find and send karas in the db that match the search expression */
bool command_find(struct lkt_state *srv, size_t c, char *cmd_args[LKT_MESSAGE_ARGS_MAX],
                  long continuation, bool (*init)(volatile sqlite3 *, struct lkt_search *));

/* Set options for the lektor such as `random`, `single`, `repeat`, etc */
enum lkt_playback_option {
    lkt_playback_option_none,
    lkt_playback_option_random,
    lkt_playback_option_single,
    lkt_playback_option_repeat,
    lkt_playback_option_consume,
    lkt_playback_option_volume,
};

bool command_set_playback_option(struct lkt_state *srv, size_t c, enum lkt_playback_option opt,
                                 char *args[LKT_MESSAGE_ARGS_MAX]);

/* Authentificate users */
bool command_password(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_user_add(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);

/* Program management control */
bool command_restart(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_kill   (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_import (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_update (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_rescan (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX], int forced);
bool command_config (struct lkt_state *srv, size_t c, char UNUSED *argv[LKT_MESSAGE_ARGS_MAX]);

/* Sticker management */
bool command_sticker_create(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_get   (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_set   (struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);
bool command_sticker_delete(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX]);

#endif /* __LKT_COMMANDS_H__ */
